import { Observable } from 'rxjs';
import { ODataCache } from './cache/index';
import { ODataCollection, ODataModel, ODataModelOptions } from './models/index';
import { ODataApiOptions } from './options';
import { ODataActionResource, ODataBatchResource, ODataEntityResource, ODataEntitySetResource, ODataFunctionResource, ODataMetadataResource, ODataNavigationPropertyResource, ODataOptions, ODataRequest, ODataResource, ODataSegment, ODataSingletonResource } from './resources/index';
import { ODataCallable, ODataEntitySet, ODataEnumType, ODataSchema, ODataStructuredType } from './schema';
import { ODataEntityService } from './services/entity';
import { ApiConfig, EdmType, Parser, QueryOption } from './types';
/**
 * Api abstraction for consuming OData services.
 */
export declare class ODataApi {
    requester?: (request: ODataRequest<any>) => Observable<any>;
    serviceRootUrl: string;
    metadataUrl: string;
    name?: string;
    version: string;
    default: boolean;
    creation: Date;
    options: ODataApiOptions;
    cache: ODataCache;
    errorHandler?: (error: any, caught: Observable<any>) => Observable<never>;
    parsers: Map<string, Parser<any>>;
    schemas: ODataSchema[];
    constructor(config: ApiConfig);
    configure(settings?: {
        requester?: (request: ODataRequest<any>) => Observable<any>;
    }): void;
    fromJSON<P, R>(json: {
        segments: ODataSegment[];
        options: {
            [name: string]: any;
        };
    }): ODataActionResource<P, R> | ODataFunctionResource<P, R>;
    fromJSON<E>(json: {
        segments: ODataSegment[];
        options: {
            [name: string]: any;
        };
    }): ODataEntityResource<E> | ODataEntitySetResource<E> | ODataNavigationPropertyResource<E> | ODataSingletonResource<E>;
    /**
     * Build a metadata resource.
     * @returns ODataMetadataResource
     */
    metadata(): ODataMetadataResource;
    /**
     * Build a batch resource.
     * @returns ODataBatchResource
     */
    batch(): ODataBatchResource;
    /**
     * Build a singleton resource.
     * @param path Name of the singleton
     * @returns
     */
    singleton<T>(path: string): ODataSingletonResource<T>;
    /**
     * Build an entity set resource.
     * @param path Name of the entity set
     * @returns
     */
    entitySet<T>(path: string): ODataEntitySetResource<T>;
    /**
     * Unbound Action
     * @param  {string} path?
     * @returns ODataActionResource
     */
    action<P, R>(path: string): ODataActionResource<P, R>;
    /**
     * Unbound Function
     * @param  {string} path?
     * @returns ODataFunctionResource
     */
    function<P, R>(path: string): ODataFunctionResource<P, R>;
    request<T>(method: string, resource: ODataResource<any>, options: ODataOptions & {
        body?: any;
        etag?: string;
        responseType?: 'arraybuffer' | 'blob' | 'json' | 'text' | 'value' | 'property' | 'entity' | 'entities';
        observe?: 'body' | 'events' | 'response';
        withCount?: boolean;
        bodyQueryOptions?: QueryOption[];
    }): Observable<any>;
    private memo;
    private findSchemaForType;
    findEnumTypeForType<T>(type: string): ODataEnumType<T> | undefined;
    findStructuredTypeForType<T>(type: string): ODataStructuredType<T> | undefined;
    findCallableForType<T>(type: string, bindingType?: string): ODataCallable<T> | undefined;
    findEntitySetForType(type: string): ODataEntitySet | undefined;
    findModelForType(type: string): typeof ODataModel | undefined;
    modelForType(type: string): typeof ODataModel;
    findCollectionForType(type: string): typeof ODataCollection | undefined;
    collectionForType(type: string): typeof ODataCollection;
    findServiceForType(type: string): typeof ODataEntityService | undefined;
    findEntitySetForEntityType(entityType: string): ODataEntitySet | undefined;
    findServiceForEntityType(entityType: string): typeof ODataEntityService | undefined;
    findEnumTypeByName<T>(name: string): ODataEnumType<T> | undefined;
    findStructuredTypeByName<T>(name: string): ODataStructuredType<T> | undefined;
    findCallableByName<T>(name: string, bindingType?: string): ODataCallable<T> | undefined;
    findEntitySetByName(name: string): ODataEntitySet | undefined;
    findModelByName(name: string): typeof ODataModel | undefined;
    findCollectionByName(name: string): typeof ODataCollection | undefined;
    findServiceByName(name: string): typeof ODataEntityService | undefined;
    parserForType<T>(type: string | EdmType, bindingType?: string): Parser<T>;
    findOptionsForType<T>(type: string): ODataModelOptions<any> | ODataModelOptions<T> | undefined;
}
