import { Subscription } from 'rxjs';
import { EntityKey, ODataEntitiesAnnotations, ODataEntityAnnotations, ODataEntityResource, ODataEntitySetResource, ODataNavigationPropertyResource, ODataPropertyResource, ODataQueryOptions, ODataQueryOptionsHandler, ODataResource, ODataSingletonResource } from '../resources';
import { ODataEntitySet, ODataStructuredType, ODataStructuredTypeFieldParser } from '../schema';
import { EdmType, Parser, ParserOptions } from '../types';
import { ODataCollection } from './collection';
import type { ODataModel } from './model';
import { EventEmitter } from '@angular/core';
export declare enum ODataModelEventType {
    Change = "change",
    Reset = "reset",
    Update = "update",
    Destroy = "destroy",
    Add = "add",
    Remove = "remove",
    Invalid = "invalid",
    Request = "request",
    Sync = "sync",
    Attach = "attach"
}
export declare class ODataModelEvent<T> {
    name: ODataModelEventType;
    value?: any;
    previous?: any;
    options?: any;
    constructor(name: ODataModelEventType, { model, collection, previous, value, attr, options, }?: {
        model?: ODataModel<T>;
        collection?: ODataCollection<T, ODataModel<T>>;
        attr?: ODataModelAttribute<any> | number;
        previous?: any;
        value?: any;
        options?: any;
    });
    bubbling: boolean;
    stopPropagation(): void;
    chain: [
        ODataModel<any> | ODataCollection<any, ODataModel<any>>,
        ODataModelAttribute<any> | number | null
    ][];
    push(model: ODataModel<any> | ODataCollection<any, ODataModel<any>>, attr: ODataModelAttribute<any> | number): ODataModelEvent<T>;
    visited(model: ODataModel<any> | ODataCollection<any, ODataModel<any>>): boolean;
    get path(): string;
    model?: ODataModel<T>;
    get currentModel(): ODataModel<any> | undefined;
    collection?: ODataCollection<T, ODataModel<T>>;
    get currentCollection(): ODataCollection<any, ODataModel<any>> | undefined;
}
export declare const BUBBLING: ODataModelEventType[];
export declare const INCLUDE_SHALLOW: {
    include_concurrency: boolean;
    include_computed: boolean;
    include_key: boolean;
};
export declare const INCLUDE_DEEP: {
    include_concurrency: boolean;
    include_computed: boolean;
    include_key: boolean;
    include_navigation: boolean;
    include_non_field: boolean;
};
export declare enum ODataModelState {
    Added = 0,
    Removed = 1,
    Unchanged = 2
}
export type ModelOptions = {
    cid?: string;
    fields: {
        [name: string]: ModelFieldOptions;
    };
};
export type ModelFieldOptions = {
    field?: string;
    parser?: ODataStructuredTypeFieldParser<any>;
    default?: any;
    required?: boolean;
    concurrency?: boolean;
    maxLength?: number;
    minLength?: number;
    min?: number;
    max?: number;
    pattern?: RegExp;
};
export declare function Model({ cid }?: {
    cid?: string;
}): <T extends new (...args: any[]) => {}>(constructor: T) => T;
export declare function ModelField({ name, ...options }?: {
    name?: string;
} & ModelFieldOptions): (target: any, key: string) => void;
export type ODataModelFieldOptions<F> = ModelFieldOptions & {
    name: string;
    field: string;
    parser: ODataStructuredTypeFieldParser<F>;
};
export declare class ODataModelField<F> {
    name: string;
    field: string;
    parser: ODataStructuredTypeFieldParser<F>;
    options: ODataModelOptions<any>;
    meta?: ODataModelOptions<any>;
    default?: any;
    required: boolean;
    concurrency: boolean;
    maxLength?: number;
    minLength?: number;
    min?: number;
    max?: number;
    pattern?: RegExp;
    parserOptions?: ParserOptions;
    constructor(options: ODataModelOptions<any>, { name, field, parser, ...opts }: ODataModelFieldOptions<F>);
    get api(): import("angular-odata").ODataApi;
    get type(): string;
    get navigation(): boolean;
    get collection(): boolean;
    annotatedValue<T>(term: string | RegExp): T | undefined;
    configure({ findOptionsForType, concurrency, options, }: {
        findOptionsForType: (type: string) => ODataModelOptions<any> | undefined;
        concurrency: boolean;
        options: ParserOptions;
    }): void;
    isKey(): boolean;
    hasReferentials(): boolean;
    get referentials(): import("../schema").ODataReferential[];
    isStructuredType(): boolean;
    structured(): ODataStructuredType<F>;
    isEnumType(): boolean;
    enum(): import("../schema").ODataEnumType<F>;
    validate(value: any, { method, navigation, }?: {
        method?: 'create' | 'update' | 'modify';
        navigation?: boolean;
    }): any;
    defaults(): any;
    deserialize(value: any, options?: ParserOptions): F;
    serialize(value: F, options?: ParserOptions): any;
    encode(value: F, options?: ParserOptions): any;
    resourceFactory<T, F>(base: ODataResource<T>): ODataNavigationPropertyResource<F> | ODataPropertyResource<F>;
    annotationsFactory<T, F>(base: ODataEntityAnnotations<T>): ODataEntityAnnotations<F> | ODataEntitiesAnnotations<F>;
    schemaFactory<T, F>(base: ODataStructuredType<T>): ODataStructuredType<F> | undefined;
    modelFactory<F>({ parent, value, reset, }: {
        parent: ODataModel<any>;
        value?: Partial<F> | {
            [name: string]: any;
        };
        reset?: boolean;
    }): ODataModel<F>;
    collectionFactory<F>({ parent, value, reset, }: {
        parent: ODataModel<any>;
        value?: Partial<F>[] | {
            [name: string]: any;
        }[];
        reset?: boolean;
    }): ODataCollection<F, ODataModel<F>>;
}
export declare class ODataModelAttribute<T> {
    private _model;
    private _field;
    private value?;
    private change?;
    private subscription?;
    events$: EventEmitter<ODataModelEvent<T>>;
    constructor(_model: ODataModel<any>, _field: ODataModelField<T>);
    get navigation(): boolean;
    get computed(): boolean | undefined;
    get concurrency(): boolean;
    get referentials(): import("../schema").ODataReferential[];
    get options(): ODataModelOptions<any>;
    get meta(): ODataModelOptions<any> | undefined;
    get name(): string;
    get fieldName(): string;
    get(): T | ODataModel<T> | ODataCollection<T, ODataModel<T>> | null | undefined;
    set(value: T | ODataModel<T> | ODataCollection<T, ODataModel<T>> | null | undefined, reset?: boolean, reparent?: boolean): boolean;
    isChanged({ include_navigation, }?: {
        include_navigation?: boolean;
    }): boolean;
    reset(): void;
    private link;
    private unlink;
}
export type ODataModelEntry<T, M extends ODataModel<T>> = {
    state: ODataModelState;
    model: M;
    key?: EntityKey<T> | {
        [name: string]: any;
    };
    subscription?: Subscription;
};
export declare class ODataModelOptions<T> {
    name: string;
    cid: string;
    base?: string;
    open?: boolean;
    private _fields;
    schema: ODataStructuredType<T>;
    entitySet?: ODataEntitySet;
    parent?: ODataModelOptions<any>;
    children: ODataModelOptions<any>[];
    constructor({ options, schema, }: {
        options: ModelOptions;
        schema: ODataStructuredType<T>;
    });
    get api(): import("angular-odata").ODataApi;
    type({ alias }?: {
        alias?: boolean;
    }): string;
    isOpenType(): boolean;
    isTypeOf(type: string): boolean;
    isModelFor(entity: T | {
        [name: string]: any;
    }): boolean;
    findChildOptions(predicate: (options: ODataModelOptions<any>) => boolean): ODataModelOptions<any> | undefined;
    configure({ options, parserForType, findOptionsForType, }: {
        options: ParserOptions;
        parserForType: (type: string) => Parser<any>;
        findOptionsForType: (type: string) => ODataModelOptions<any> | undefined;
    }): void;
    fields({ include_navigation, include_parents, }: {
        include_parents: boolean;
        include_navigation: boolean;
    }): ODataModelField<any>[];
    field<F>(name: keyof T | string): ODataModelField<F>;
    findField<F>(name: keyof T | string): ODataModelField<F> | undefined;
    addField<F>(name: string, options: ModelFieldOptions): ODataModelField<F>;
    tsToEdm: Record<string, EdmType>;
    private modelFieldFactory;
    attach(self: ODataModel<T>, resource: ODataEntityResource<T> | ODataNavigationPropertyResource<T> | ODataPropertyResource<T> | ODataSingletonResource<T>): void;
    static chain(child: ODataModel<any> | ODataCollection<any, ODataModel<any>>): [
        ODataModel<any> | ODataCollection<any, ODataModel<any>>,
        ODataModelField<any> | null
    ][];
    static resource<T>(child: ODataModel<T> | ODataCollection<T, ODataModel<T>>): ODataResource<T>;
    collectionResourceFactory(query?: ODataQueryOptions<T>): ODataEntitySetResource<T> | ODataNavigationPropertyResource<T> | ODataPropertyResource<T> | undefined;
    modelResourceFactory(query?: ODataQueryOptions<T>): ODataEntityResource<T> | ODataNavigationPropertyResource<T> | ODataPropertyResource<T> | ODataSingletonResource<T> | undefined;
    entityResource(self: ODataModel<T>): ODataResource<T>;
    bind(self: ODataModel<T>, { parent, resource, annots, }?: {
        parent?: [
            ODataModel<any> | ODataCollection<any, ODataModel<any>>,
            ODataModelField<any> | null
        ];
        resource?: ODataResource<T>;
        annots?: ODataEntityAnnotations<T>;
    }): void;
    query(self: ODataModel<T>, resource: ODataEntityResource<T> | ODataPropertyResource<T> | ODataNavigationPropertyResource<T> | ODataSingletonResource<T>, func: (q: ODataQueryOptionsHandler<T>, s?: ODataStructuredType<T>) => void): ODataModel<T>;
    resolveKey(value: ODataModel<T> | T | {
        [name: string]: any;
    }, { field_mapping, resolve, single, }?: {
        field_mapping?: boolean;
        resolve?: boolean;
        single?: boolean;
    }): EntityKey<T> | {
        [name: string]: any;
    } | undefined;
    resolveReferential(value: ODataModel<T> | T | {
        [name: string]: any;
    } | null, attr: ODataModelAttribute<any> | ODataModelField<any>, { field_mapping, resolve, single, }?: {
        field_mapping?: boolean;
        resolve?: boolean;
        single?: boolean;
    }): {
        [name: string]: any;
    } | null | undefined;
    resolveReferenced(value: ODataModel<T> | T | {
        [name: string]: any;
    } | null, attr: ODataModelAttribute<any> | ODataModelField<any>, { field_mapping, resolve, single, }?: {
        field_mapping?: boolean;
        resolve?: boolean;
        single?: boolean;
    }): {
        [name: string]: any;
    } | null | undefined;
    validate(self: ODataModel<T>, { method, navigation, }?: {
        method?: 'create' | 'update' | 'modify';
        navigation?: boolean;
    }): {
        [name: string]: string[];
    } | undefined;
    defaults(): T | {
        [name: string]: any;
    } | undefined;
    hasChanged(self: ODataModel<T>, { include_navigation }?: {
        include_navigation?: boolean;
    }): boolean;
    hasKey(self: ODataModel<T>): boolean;
    withResource<R, M extends ODataModel<T>>(self: M, resource: ODataEntityResource<T> | ODataPropertyResource<T> | ODataNavigationPropertyResource<T> | ODataSingletonResource<T>, ctx: (model: M) => R): R;
    asEntity<R, M extends ODataModel<T>>(self: M, ctx: (model: M) => R): R;
    toEntity(self: ODataModel<T>, { client_id, include_navigation, include_concurrency, include_computed, include_key, include_non_field, changes_only, field_mapping, chain, }?: {
        client_id?: boolean;
        include_navigation?: boolean;
        include_concurrency?: boolean;
        include_computed?: boolean;
        include_key?: boolean;
        include_non_field?: boolean;
        changes_only?: boolean;
        field_mapping?: boolean;
        chain?: (ODataModel<any> | ODataCollection<any, ODataModel<any>>)[];
    }): T | {
        [name: string]: any;
    };
    reset(self: ODataModel<T>, { name, silent }?: {
        name?: string;
        silent?: boolean;
    }): void;
    assign(self: ODataModel<T>, entity: Partial<T> | {
        [name: string]: any;
    }, { reset, reparent, silent, }?: {
        reset?: boolean;
        reparent?: boolean;
        silent?: boolean;
    }): void;
    static isModel(obj: any): boolean;
    static isCollection(obj: any): boolean;
    private _updateCollection;
    private _updateModel;
    get<F>(self: ODataModel<T>, field: ODataModelField<F> | string): F | ODataModel<F> | ODataCollection<F, ODataModel<F>> | null | undefined;
    set<F>(self: ODataModel<T>, field: ODataModelField<F> | string, value: F | F[] | {
        [name: string]: any;
    } | {
        [name: string]: any;
    }[] | ODataModel<F> | ODataCollection<F, ODataModel<F>> | null, { type }?: {
        type?: string;
    }): boolean;
    private _link;
}
