import { Observable } from 'rxjs';
import { ODataApi } from '../api';
import { ODataCollection, ODataModel } from '../models';
import { ODataStructuredType } from '../schema';
import { ODataSchemaElement } from '../schema/element';
import { ParserOptions, QueryOption } from '../types';
import { ODataPathSegments, ODataPathSegmentsHandler } from './path';
import { ODataQueryOptions, ODataQueryOptionsHandler, QueryCustomType } from './query';
import { ODataEntitiesAnnotations, ODataEntityAnnotations } from './responses/index';
import { ODataOptions } from './types';
export type EntityKey<T> = {
    readonly [P in keyof T]?: T[P];
} | QueryCustomType | string | number;
export declare class ODataResource<T> {
    api: ODataApi;
    schema?: ODataSchemaElement;
    protected pathSegments: ODataPathSegments;
    protected queryOptions: ODataQueryOptions<T>;
    constructor(api: ODataApi, { segments, query, schema, }?: {
        segments?: ODataPathSegments;
        query?: ODataQueryOptions<T>;
        schema?: ODataSchemaElement;
    });
    /**
     * @returns string The type of the resource
     */
    type(): string | undefined;
    /**
     * @returns string The type of the return
     */
    returnType(): string | undefined;
    /**
     * @returns string All covered types of the resource
     */
    types(): string[];
    /**
     * @returns boolean The resource has key ?
     */
    hasKey(): boolean;
    hasEntityKey(): boolean;
    clearKey(): void | undefined;
    asModel<M extends ODataModel<T>>(entity?: Partial<T> | {
        [name: string]: any;
    }, { annots }?: {
        annots?: ODataEntityAnnotations<T>;
    }): M;
    asCollection<M extends ODataModel<T>, C extends ODataCollection<T, M>>(entities?: Partial<T>[] | {
        [name: string]: any;
    }[], { annots }?: {
        annots?: ODataEntitiesAnnotations<T>;
    }): C;
    isSubtypeOf(other: ODataResource<any>): boolean;
    isSupertypeOf(other: ODataResource<any>): boolean;
    isEqualTo(other: ODataResource<any>, test?: 'path' | 'params'): boolean;
    pathAndParams(escape?: boolean): [string, {
        [name: string]: any;
    }];
    endpointUrl(params?: boolean): string;
    toString(): string;
    clone(): ODataResource<T>;
    private __parser;
    deserialize(value: any, options?: ParserOptions): any;
    serialize(value: any, options?: ParserOptions): any;
    encode(value: any, options?: ParserOptions): any;
    toJSON(): {
        segments: any[];
        options: {};
    };
    cloneSegments(): ODataPathSegments;
    clearQuery(): void;
    cloneQuery<P>(): ODataQueryOptions<P>;
    /**
     * Handle the path segments of the resource
     * Create an object handler for mutate the path segments of the resource
     * @param f Function context for handle the segments
     * @returns ODataActionResource
     */
    segment(f: (q: ODataPathSegmentsHandler<T>, s?: ODataStructuredType<T>) => void): this;
    /**
     * Handle the query options of the resource
     * Create an object handler for mutate the query options of the resource
     * @param f Function context for handle the query options
     */
    query(f: (q: ODataQueryOptionsHandler<T>, s?: ODataStructuredType<T>) => void): this;
    static resolveKey<T>(value: any, schema?: ODataStructuredType<T>): EntityKey<T> | undefined;
    protected resolveKey(value: any): EntityKey<T> | undefined;
    protected get(options?: ODataOptions & {
        etag?: string;
        responseType?: 'arraybuffer' | 'blob' | 'json' | 'text' | 'value' | 'property' | 'entity' | 'entities';
        withCount?: boolean;
        bodyQueryOptions?: QueryOption[];
    }): Observable<any>;
    protected post(body: any, options?: ODataOptions & {
        responseType?: 'arraybuffer' | 'blob' | 'json' | 'text' | 'value' | 'property' | 'entity' | 'entities';
        withCount?: boolean;
    }): Observable<any>;
    protected put(body: any, options?: ODataOptions & {
        etag?: string;
        responseType?: 'arraybuffer' | 'blob' | 'json' | 'text' | 'value' | 'property' | 'entity' | 'entities';
        withCount?: boolean;
    }): Observable<any>;
    protected patch(body: any, options?: ODataOptions & {
        etag?: string;
        responseType?: 'arraybuffer' | 'blob' | 'json' | 'text' | 'value' | 'property' | 'entity' | 'entities';
        withCount?: boolean;
    }): Observable<any>;
    protected delete(options?: ODataOptions & {
        etag?: string;
        responseType?: 'arraybuffer' | 'blob' | 'json' | 'text' | 'value' | 'property' | 'entity' | 'entities';
        withCount?: boolean;
    }): Observable<any>;
}
